@@ -442,11 +442,11 @@ ReadCounts=function(file, design=c(Design$Condition,Design$Replicate),classify.g
   url=NULL
 
   if (suppressWarnings(requireNamespace("RCurl",quietly = TRUE))) {
-    if (RCurl::url.exists(prefix)) {
+    if (RCurl::url.exists(prefix,timeout.ms=1000)) {
       url=prefix
-    } else if (RCurl::url.exists(paste0(prefix,".tsv"))) {
+    } else if (RCurl::url.exists(paste0(prefix,".tsv"),timeout.ms=1000)) {
       url=paste0(prefix,".tsv")
-    } else if (RCurl::url.exists(paste0(prefix,".tsv.gz"))) {
+    } else if (RCurl::url.exists(paste0(prefix,".tsv.gz"),timeout.ms=1000)) {
       url=paste0(prefix,".tsv.gz")
     } else {
       url=NULL
@@ -624,11 +624,11 @@ ReadFeatureCounts=function(file, design=c(Design$Condition,Design$Replicate),cla
 
 
   if (suppressWarnings(requireNamespace("RCurl",quietly = TRUE))) {
-    if (RCurl::url.exists(prefix)) {
+    if (RCurl::url.exists(prefix,timeout.ms=1000)) {
       url=prefix
-    } else if (RCurl::url.exists(paste0(prefix,".tsv"))) {
+    } else if (RCurl::url.exists(paste0(prefix,".tsv"),timeout.ms=1000)) {
       url=paste0(prefix,".tsv")
-    } else if (RCurl::url.exists(paste0(prefix,".tsv.gz"))) {
+    } else if (RCurl::url.exists(paste0(prefix,".tsv.gz"),timeout.ms=1000)) {
       url=paste0(prefix,".tsv.gz")
     } else {
       url=NULL
@@ -763,7 +763,6 @@ GetTableQC=function(data,name,stop.if.not.exist=TRUE) {
 }
 
 
-
 #' Read the output of GRAND-SLAM 3.0 into a grandR object.
 #'
 #' Metabolic labeling - nucleotide conversion RNA-seq data (such as generated by SLAM-seq,TimeLapse-seq or TUC-seq)
@@ -772,6 +771,8 @@ GetTableQC=function(data,name,stop.if.not.exist=TRUE) {
 #' This function directly reads the output of GRAND-SLAM 3.0 into a grandR object.
 #'
 #' @param prefix the prefix used to call GRAND-SLAM
+#' @param pseudobulk.name the pseudobulkName used to call GRAND-SLAM
+#' @param targets.name the targetsName used to call GRAND-SLAM
 #' @param design Either a design vector (see details), or a data.frame providing metadata for all columns (samples/cells),
 #' or a function that is called with the condition name vector and is supposed to return this data.frame. if NULL, a
 #' library,sample,barcode design is used for sparse data, and a condition,replicate design for dense data
@@ -779,7 +780,7 @@ GetTableQC=function(data,name,stop.if.not.exist=TRUE) {
 #' @param estimator which estimator to use (one of Binom,TbBinom,TbBinomShape)
 #' @param classify.genes A function that is used to add the \emph{type} column to the gene annotation table, always a call to \link{ClassifyGenes}
 #' @param read.posterior also read the posterior parameters alpha and beta? if NULL, TRUE for dense data, FALSE for sparse data
-#' @param rename.sample function that is applied to each sample name before parsing (or NULL)
+#' @param rename.sample function that is applied to each sample name before parsing (or NULL; use the \link{Renamer} function)
 #' @param verbose Print status updates
 #'
 #' @return A grandR object containing the read counts, NTRs, information on the NTR posterior distribution (alpha,beta)
@@ -804,13 +805,15 @@ GetTableQC=function(data,name,stop.if.not.exist=TRUE) {
 #' @details Sometimes you might have forgotten to name all samples consistently (or you simply messed something up).
 #' In this case, the rename.sample parameter can be handy (e.g. to rename a particular misnamed sample).
 #'
-#' @seealso \link{ReadGRAND},\link{ClassifyGenes},\link{MakeColdata},\link{DesignSemantics}
+#' @seealso \link{ReadGRAND},\link{ClassifyGenes},\link{MakeColdata},\link{DesignSemantics},\link{Renamer}
 #'
 #'
 #' @export
 #'
 #' @concept load
 ReadGRAND3=function(prefix,
+                    pseudobulk.name = NULL,
+                    targets.name = "targets",
                     design=NULL,
                     label="4sU",
                     estimator="Binom",
@@ -822,43 +825,45 @@ ReadGRAND3=function(prefix,
   if (length(estimator)!=1 || !estimator %in% c("Binom","TbBinom","TbBinomShape")) stop("Invalid estimator!")
 
   # TODO: do not use read.delim or .csv, nor file.exists (to allow using urls!)
-  isSparse=function(prefix) !file.exists(paste0(prefix,".targets/data.tsv.gz")) & !file.exists(prefix)
+  # read.table(gzfile(url), sep = "\t", header = TRUE) can replace read.delim and handle urls
 
-  if (isSparse(prefix)) {
+  build_prefix = function() {
+    if (file.exists(prefix)) return(prefix)
+    prefix = paste0(c(prefix, if (!is.null(pseudobulk.name)) "pseudobulk", targets.name, pseudobulk.name), collapse = ".")
+    f = paste0(prefix, "/data.tsv.gz")
+    if (file.exists(f)) f else prefix
+  }
+  prefix=build_prefix()
+
+  isSparse=function() !grepl("/data\\.tsv\\.gz$", prefix)
+
+  if (isSparse()) {
     if (is.null(design)) design=c(Design$Library,Design$Sample,Design$Barcode)
     if (is.null(read.posterior)) read.posterior=FALSE
-    ReadGRAND3_sparse(prefix=prefix,design=design,label=label,estimator=estimator,classify.genes=classify.genes,read.posterior=read.posterior,rename.sample = rename.sample,verbose=verbose)
+    ReadGRAND3_sparse(prefix=prefix,design=design,label=label,estimator=estimator,classify.genes=classify.genes,read.posterior=read.posterior,rename.sample=rename.sample,verbose=verbose)
   } else {
     if (is.null(design)) design=c(Design$Condition,Design$Replicate)
     if (is.null(read.posterior)) read.posterior=TRUE
-    ReadGRAND3_dense(prefix=prefix,design=design,label=label,estimator=estimator,classify.genes=classify.genes,read.posterior=read.posterior,rename.sample = rename.sample,verbose=verbose)
+    ReadGRAND3_dense(prefix=prefix,design=design,label=label,estimator=estimator,classify.genes=classify.genes,read.posterior=read.posterior,rename.sample=rename.sample,verbose=verbose)
   }
 }
-ReadGRAND3_sparse=function(prefix,
-                           design=c(Design$Library,Design$Sample,Design$Barcode),
-                           label="4sU",
-                           estimator="Binom",
-                           classify.genes=ClassifyGenes(),
-                           read.posterior=FALSE,
-                           rename.sample=NULL,
-                           verbose=FALSE) {
 
-  cols=readLines(paste0(prefix, ".targets/barcodes.tsv.gz"))
+
+ReadGRAND3_sparse=function(prefix,design=c(Design$Library,Design$Sample,Design$Barcode),label="4sU",estimator="Binom",classify.genes=ClassifyGenes(),read.posterior=FALSE,rename.sample=NULL,verbose=FALSE) {
+
+  cols=readLines(paste0(prefix,"/barcodes.tsv.gz"))
   if (!is.null(rename.sample)) cols=sapply(cols,rename.sample) # let's use sapply, we have no idea whether the function works with vectorization
 
   conds=strsplit(cols,".",fixed=TRUE)[[1]]
 
   if (length(conds)!=length(design)) stop(paste0("Design parameter is incompatible with input data: ",paste(conds,collapse=".")))
 
-
-  if (verbose) cat("Reading count matrix...\n")
-  count=Matrix::readMM(paste0(prefix, ".targets/matrix.mtx.gz"))
-  gene.info=utils::read.delim(paste0(prefix, ".targets/features.tsv.gz"),header = FALSE,stringsAsFactors = FALSE)
-  if (ncol(gene.info)==4) gene.info$Length=1
+  gene.info=utils::read.delim(paste0(prefix,"/features.tsv.gz"),header=FALSE,stringsAsFactors=FALSE)
+  if (ncol(gene.info)==4) gene.info$Length=1 #TODO: what does this indicate, i.e., what does Length mean?
   gene.info=setNames(gene.info,c("Gene","Symbol","Mode","Category","Length"))
 
-  gene.info$Gene = check.and.make.unique(gene.info$Gene,label="gene names")
-  gene.info$Symbol = check.and.make.unique(gene.info$Symbol,ref=gene.info$Gene,label="gene symbols",ref.label = "gene names")
+  gene.info$Gene=check.and.make.unique(gene.info$Gene,label="gene names")
+  gene.info$Symbol=check.and.make.unique(gene.info$Symbol,ref=gene.info$Gene,label="gene symbols",ref.label="gene names")
 
   #if (anyDuplicated(gene.info$Gene)) {
   #  dupp=table(gene.info$Gene)
@@ -884,45 +889,75 @@ ReadGRAND3_sparse=function(prefix,
 #    gene.info$Symbol=make.names(gene.info$Symbol)
 #  }
 
-  colnames(count)=cols
-  rownames(count)=gene.info$Gene
-  re=list()
-  re$count=count
+ re=list()
+
+  make_MM=function(path) {
+    mm=Matrix::readMM(path)
+    colnames(mm)=cols
+    rownames(mm)=gene.info$Gene
+    return(mm)
+  }
+
+  if (verbose) cat("Reading count matrix...\n")
+  re$count=make_MM(paste0(prefix,"/matrix.mtx.gz"))
 
   if (verbose) cat("Reading NTRs...\n")
-  ntr=Matrix::readMM(sprintf("%s.targets/%s.%s.ntr.mtx.gz",prefix,label,estimator))
-  colnames(ntr)=cols
-  rownames(ntr)=gene.info$Gene
-  re$ntr=ntr
+  re$ntr=make_MM(sprintf("%s/%s.%s.ntr.mtx.gz",prefix,label,estimator))
 
   if (estimator=="TbBinomShape") {
 
     if (verbose) cat("Reading LLRs...\n")
-    llr <- Matrix::readMM(paste0(prefix,".targets/4sU.llr.mtx.gz"))
-    colnames(llr)=cols
-    rownames(llr)=gene.info$Gene
-    re$llr=llr
+    re$llr=make_MM(paste0(prefix,"/4sU.llr.mtx.gz"))
+
+    # not all data we are working with already produced this output
+    if (verbose) cat("Reading LLs...\n")
+    ll.file <- paste0(prefix,"/4sU.ll.mtx.gz")
+    if (file.exists(ll.file)) re$ll=make_MM(ll.file) else message(sprintf("%s not found - skipping", ll.file))
 
     if (verbose) cat("Reading Shapes...\n")
-    shape <- Matrix::readMM(paste0(prefix,".targets/4sU.shape.mtx.gz"))
-    colnames(shape)=cols
-    rownames(shape)=gene.info$Gene
-    re$shape=shape
-
+    re$shape=make_MM(paste0(prefix,"/4sU.shape.mtx.gz"))
   }
 
-  if (read.posterior && file.exists(sprintf("%s.targets/%s.%s.alpha.mtx.gz",prefix,label,estimator)) && file.exists(sprintf("%s.targets/%s.%s.beta.mtx.gz",prefix,label,estimator))) {
-    if (verbose) cat("Reading posterior beta parameters...\n")
-    alpha=Matrix::readMM(sprintf("%s.targets/%s.%s.alpha.mtx.gz",prefix,label,estimator))
-    colnames(alpha)=cols
-    rownames(alpha)=gene.info$Gene
-    re$alpha=alpha
+  # if (read.posterior && file.exists(sprintf("%s/%s.%s.alpha.mtx.gz",prefix,label,estimator)) && file.exists(sprintf("%s/%s.%s.beta.mtx.gz",prefix,label,estimator))) {
+  #   if (verbose) cat("Reading posterior beta parameters...\n")
+  #   re$alpha=make_MM(sprintf("%s/%s.%s.alpha.mtx.gz",prefix,label,estimator))
+  #   re$beta=make_MM(sprintf("%s/%s.%s.beta.mtx.gz",prefix,label,estimator))
+  # }
+  if (read.posterior) {
+    alpha_files <- list.files(prefix, pattern = sprintf("^%s\\.%s\\.(alpha[^\\.]*)\\.mtx\\.gz$", label, estimator), full.names = TRUE)
+    beta_files  <- list.files(prefix, pattern = sprintf("^%s\\.%s\\.(beta[^\\.]*)\\.mtx\\.gz$", label, estimator), full.names = TRUE)
+    integral_file <- file.path(prefix, sprintf("%s.%s.integral.mtx.gz", label, estimator))
+    mix_file      <- file.path(prefix, sprintf("%s.%s.mix.mtx.gz", label, estimator))
 
-    beta=Matrix::readMM(sprintf("%s.targets/%s.%s.beta.mtx.gz",prefix,label,estimator))
-    colnames(beta)=cols
-    rownames(beta)=gene.info$Gene
-    re$beta=beta
+    if (length(alpha_files) > 0 && length(beta_files) > 0) {
+      if (verbose) cat("Reading posterior beta parameters from multiple files...\n")
+
+      for (f in alpha_files) {
+        name <- sub(sprintf("^%s/%s\\.%s\\.(alpha[^\\.]*)\\.mtx\\.gz$", prefix, label, estimator), "\\1", f)
+        re[[name]] <- make_MM(f)
+      }
+
+      for (f in beta_files) {
+        name <- sub(sprintf("^%s/%s\\.%s\\.(beta[^\\.]*)\\.mtx\\.gz$", prefix, label, estimator), "\\1", f)
+        re[[name]] <- make_MM(f)
+      }
+
+      if (file.exists(integral_file)) {
+        if (verbose) cat("Reading posterior integral matrix...\n")
+        re$integral <- make_MM(integral_file)
+      } else if (verbose) {
+        cat("Integral file not found: ", integral_file, "\n")
+      }
+
+      if (file.exists(mix_file)) {
+        if (verbose) cat("Reading posterior mix matrix...\n")
+        re$mix <- make_MM(mix_file)
+      } else if (verbose) {
+        cat("Mix file not found: ", mix_file, "\n")
+      }
+    }
   }
+
   if (is.null(gene.info$Mode)) {
     gene.info$Mode=gsub(".*\\(","",gsub(")","",gene.info$Category,fixed=TRUE))
     gene.info$Mode=factor(gene.info$Mode,levels=unique(gene.info$Mode))
@@ -931,21 +966,21 @@ ReadGRAND3_sparse=function(prefix,
   gene.info$Type=classify.genes(gene.info)
 
   coldata=MakeColdata(cols,design)
-  # use make coldata instead. add no4sU column!
+   # use make coldata instead. add no4sU column!
   #coldata=data.frame(Name=cols)
   #spl=strsplit(as.character(coldata$Name),".",fixed=TRUE)
   #for (i in 1:length(design)) coldata=cbind(coldata,factor(sapply(spl,function(v) v[i]),levels=unique(sapply(spl,function(v) v[i]))))
   #names(coldata)[-1]=design
   #rownames(coldata)=coldata$Name
-
-  coldata$no4sU=Matrix::colSums(ntr)==0
+  coldata$no4sU=Matrix::colSums(re$ntr)==0
 
   re=grandR(prefix=prefix,gene.info=gene.info,slots=re,coldata=coldata,metadata=list(`GRAND-SLAM version`=3,Output="sparse"))
   DefaultSlot(re)="count"
-  re
+  return(re)
 }
 
 
+
 ReadGRAND3_dense=function(prefix,
                           design=c(Design$Condition,Design$Replicate),
                           label="4sU",
@@ -956,16 +991,18 @@ ReadGRAND3_dense=function(prefix,
                           verbose=FALSE) {
   annotations=c("Gene","Symbol","Category","Length")
   slots=c("count","ntr","alpha","beta")
-  names(slots)=c("Read count",sprintf("%s %s %s",label,estimator,c("NTR MAP","alpha","beta")))
+  names(slots)=c("Read count", sprintf("%s %s %s",label,estimator,c("NTR MAP","alpha","beta")))
   if (!read.posterior) slots=slots[1:2]
-  if (estimator=="TbBinomShape") slots=c(slots,Shape="shape",LLR="llr")
+  if (estimator=="TbBinomShape") slots=c(slots,Shape="shape",LLR="llr",LL="ll")
+  # TODO: not all data we are working with already produced LL output, but it should be inculded in the next release
 
-  re=read.grand.internal(description="GRAND-SLAM 3.0 dense data",prefix = prefix, design = design, slots=slots, annotations=annotations,classify.genes = classify.genes,rename.sample = rename.sample,verbose = verbose)
+  re=read.grand.internal(description="GRAND-SLAM 3.0 dense data",prefix=prefix,design=design,slots=slots,annotations=annotations,classify.genes=classify.genes,rename.sample=rename.sample,verbose=verbose)
   re$metadata=c(re$metadata,list(`GRAND-SLAM version`=3,Output="dense"))
-  re
+  return(re)
 }
 
 
+
 #' Read sparse new/total matrices
 #'
 #' This function can be used to load matrix market data in case genes were quantified by
@@ -1049,17 +1086,10 @@ try.file = function(prefix, possible.suffixes=c("",".tsv",".tsv.gz",".targets/da
   for (suffix in possible.suffixes) {
     if (file.exists(paste0(prefix,suffix))) return(list(file=paste0(prefix,suffix),prefix=cut.suffix(prefix),callback=do.callback))
   }
-
+  #prefix
 
   if (suppressWarnings(requireNamespace("RCurl",quietly = TRUE))) {
-    url=NULL
-    for (suffix in possible.suffixes) {
-      if (RCurl::url.exists(paste0(prefix,suffix))) {
-        url=paste0(prefix,suffix)
-        break
-      }
-    }
-
+    url=prefix
 
     if (!is.null(url)) {
       fn=gsub(".*/","",gsub("\\?.*","",url))
@@ -1137,6 +1167,7 @@ read.grand.internal=function(prefix, design=c(Design$Condition,Design$Replicate)
   #
   # if (!file.exists(file) && !hascurl) stop("File not found; If you want to access non-local files directly, please install the RCurl package!")
   # if (!file.exists(file)) stop("File not found!")
+
   tfile=try.file(prefix,verbose=verbose)
   file=tfile$file
   prefix=tfile$prefix
@@ -1214,12 +1245,17 @@ read.grand.internal=function(prefix, design=c(Design$Condition,Design$Replicate)
   gene.info$Type=classify.genes(gene.info)
 
 
-  re=list()
+    re=list()
   for (n in names(slots)) {
     cols=intersect(paste0(conds," ",n),names(data))
     a=tomat(data[,cols],data$Gene,cols)
     if (ncol(a)==0 && n=="Conversions") stop("Columns for conversions are not available in GRAND-SLAM output; rerun gedi -e Slam with parameter -full!")
-    re[[slots[n]]]=a
+    if (ncol(a)==0 && n=="LL") {
+      warning("Column LL is not available in GRAND3 output; rerun gedi -e Grand3!")
+      re[[slots[n]]]=NULL # explicitly set to NULL; is zero column matrix otherwise and does not pass check during grandR object creation
+    } else {
+      re[[slots[n]]]=a
+    }
   }
   #re$count=tomat(data[,count],data$Gene,names(data)[count])
   ##	re$tpm=comp.tpm(re$count,gene.info$Length)
@@ -1246,6 +1282,7 @@ read.grand.internal=function(prefix, design=c(Design$Condition,Design$Replicate)
     re$beta=correctmat(re$beta)
     if (!is.null(re$shape)) re$shape=correctmat(re$shape)
     if (!is.null(re$llr)) re$llr=correctmat(re$llr)
+    if (!is.null(re$ll)) re$ll=correctmat(re$ll)
   }
 
   checknames=function(n,a){
@@ -1255,7 +1292,7 @@ read.grand.internal=function(prefix, design=c(Design$Condition,Design$Replicate)
     if (!all(rownames(a)==gene.info$Gene)) stop(sprintf("Row names do not match for %s!",n))
   }
 
-  for (slotname in names(re)) checknames(slotname,re[[slotname]])
+  for (slotname in names(re)) checknames(slotname,re[[slotname, exact = TRUE]])
 
   coldata$no4sU=no4sU.cols
 
