@@ -392,9 +392,72 @@ logLik_MixMat=function(m,fun,...) {
 	re
 }
 
-fit.ntr=function(mixmat,par,beta.approx=FALSE,conversion.reads=FALSE,plot=FALSE) {
+fit.ntr.ll=function(ll0,ll1,c,beta.approx=FALSE,plot=FALSE, prior = c(1,1)) {
+  start <- 0
+  optfun = function(p) {
+    ldb = lse(ll0+log(1-p),ll1+log(p))
+    sum(c * ldb) - start
+  }
+  opt <- optimize(optfun, maximum = TRUE, lower = 0, upper = 1)
+
+  ll_0 <- optfun(0)
+  ll_1 <- optfun(1)
+
+  if (ll_0 > opt$objective) {
+    opt$maximum <- 0
+    opt$objective <- ll_0
+  }
+  else if (ll_1 > opt$objective) {
+    opt$maximum <- 1
+    opt$objective <- ll_1
+  }
+
+  if (!beta.approx) {
+    return(opt$maximum)
+  }
+
+  start=opt$objective+start
+
+  left=if(optfun(0)>log(1E-3)) 0 else uniroot(function(x) optfun(x)-log(1E-3),c(0,opt$maximum))$root
+  right=if(optfun(1)>log(1E-3)) 1 else uniroot(function(x) optfun(x)-log(1E-3),c(opt$maximum,1))$root
+
+  x=seq(left,right,length.out=100)
   start=0
-  optfun=function(p) logLik_MixMat(mixmat,dbinommix,ntr=p,p.err=par$p.err,p.conv=par$p.conv)-start
+  fs = sapply(x,optfun)-log(length(x)-1);
+  fs2=rep(NA,length(fs))
+
+  fs[1]=fs[1]-log(2);
+  fs2[1] = fs[1];
+
+  for (i in 2:length(fs)) {
+    fs2[i] =  lse(fs[i-1],fs[i]-log(2));
+    fs[i] = lse(fs[i-1],fs[i]);
+  }
+  fs2=exp(fs2-fs2[length(fs2)])
+  ooptfun = function(par) sum((pbeta(x,par[1],par[2])-fs2)^2)
+  shapes=constrOptim(c(alpha=3,beta=3),ooptfun,grad=NULL,ui=cbind(c(1,0),c(0,1)),ci=c(0,0))$par
+
+  logpost <- sapply(x, optfun)
+  #optfun_safe <- Vectorize(optfun)
+  #int_part <- integrate(function(p) exp(optfun_safe(p)), lower = left, upper = right)$value
+  #integral1 <- start + log(int_part)
+  log_int <- lsse(logpost) + log(x[2]-x[1])
+
+  integral <- start + log_int
+
+  if (plot) {
+    plot(x,fs2,xlab="ntr",ylab="Cumulative freq",type='l')
+    graphics::lines(x,pbeta(x,shapes[1],shapes[2]),col='red')
+    graphics::legend("topleft",legend=c("Actual distribution","Beta approximation"),fill=c("black","red"))
+  }
+
+  c(ntr=opt$maximum,shapes, integral = integral)
+}
+
+
+fit.ntr=function(mixmat,par,beta.approx=FALSE,conversion.reads=FALSE,plot=FALSE, prior = c(1,1)) {
+  start=0
+  optfun=function(p) dbeta(p,prior[1],prior[2],log=TRUE)+logLik_MixMat(mixmat,dbinommix,ntr=p,p.err=par$p.err,p.conv=par$p.conv)-start
   start=optfun(par$ntr)
   opt=optimize(optfun,maximum=TRUE,lower=0,upper=1)
 
@@ -432,7 +495,16 @@ fit.ntr=function(mixmat,par,beta.approx=FALSE,conversion.reads=FALSE,plot=FALSE)
     fs[i] = lse(fs[i-1],fs[i]);
   }
   fs2=exp(fs2-fs2[length(fs2)])
-  shapes=constrOptim(c(alpha=3,beta=3),function(par) sum((pbeta(x,par[1],par[2])-fs2)^2),grad=NULL,ui=cbind(c(1,0),c(0,1)),ci=c(0,0))$par
+  ooptfun = function(par) sum((pbeta(x,par[1],par[2])-fs2)^2)
+  shapes=constrOptim(c(alpha=3,beta=3),ooptfun,grad=NULL,ui=cbind(c(1,0),c(0,1)),ci=c(0,0))$par
+
+  logpost <- sapply(x, function(p) logLik_MixMat(mixmat,dbinommix,ntr=p,p.err=par$p.err,p.conv=par$p.conv))
+  #optfun_safe <- Vectorize(optfun)
+  #int_part <- integrate(function(p) exp(optfun_safe(p)), lower = left, upper = right)$value
+  #integral1 <- start + log(int_part)
+  log_int <- lsse(logpost) + log(x[2]-x[1])
+
+  integral <- start + log_int
 
   if (plot) {
     plot(x,fs2,xlab="ntr",ylab="Cumulative freq",type='l')
@@ -440,7 +512,7 @@ fit.ntr=function(mixmat,par,beta.approx=FALSE,conversion.reads=FALSE,plot=FALSE)
     graphics::legend("topleft",legend=c("Actual distribution","Beta approximation"),fill=c("black","red"))
   }
 
-  if (conversion.reads) c(ntr=opt$maximum,shapes,conversion.reads=sum(mixmat)-sum(mixmat[0,])) else c(ntr=opt$maximum,shapes)
+  if (conversion.reads) c(ntr=opt$maximum,shapes,conversion.reads=sum(mixmat)-sum(mixmat[0,]),integral=integral) else c(ntr=opt$maximum,shapes, integral = integral)
 }
 
 binom.optim=function(mixmat,par,fix=c(F,F,F)) {
@@ -490,8 +562,8 @@ tbbinom.optim=function(mixmat,par,fix=c(F,F,F,F)) {
     opt=optimize(optfun,maximum=TRUE,lower=l[!fix],upper=u[!fix])
     re=list(convergence=0,par=setNames(opt$maximum,names(pp)[!fix]),value=opt$objective)
   } else {
-    ui=cbind(c(1,-1,0,0,0,0),c(0,0,1,-1,0,0),c(0,0,0,0,1,-1),c(0,0,0,0,0,0))
-    ci=c(0,-1,0,-MAX_ERR,0,-1)
+    ui=cbind(c(1,-1,0,0,0,0,0,0),c(0,0,1,-1,0,0,0,0),c(0,0,0,0,1,-1,0,0),c(0,0,0,0,0,0,1,-1))
+    ci=c(0,-1,0,-MAX_ERR,0,-1,-2,-5)
     ui=ui[,!fix,drop=FALSE]
     use=apply(ui!=0,1,any)
     ui=ui[use,,drop=FALSE]
@@ -573,7 +645,7 @@ tbbinom.optim2=function(mixmat,par,fix=c(F,F)) {
 }
 
 
-fit.MixMat=function(mixmat,par=default.model.par,type=c("binom","tubinom","tbbinom"),fix=rep(FALSE,4)) {
+fit.MixMat=function(mixmat,par=default.model.par,type=c("binom","tubinom","tbbinom"),fix=rep(FALSE,4),error.on.nonconverge=TRUE) {
 	if (is.list(mixmat)) return(lapply(mixmat,fit.MixMat,par=par,type=type,fix=fix))
 
 	opti=switch(type[1],
@@ -581,7 +653,7 @@ fit.MixMat=function(mixmat,par=default.model.par,type=c("binom","tubinom","tbbin
 			tubinom=tbbinom.optim(mixmat,model.par(ntr=par$ntr,p.err=par$p.err,p.mconv=par$p.mconv,0),fix=c(fix[1:3],FALSE)),
 			tbbinom=tbbinom.optim(mixmat,par,fix)
 		)
-	if (opti$convergence!=0) stop("Did not converge!")
+	if (error.on.nonconverge && opti$convergence!=0) stop("Did not converge!")
 
 	opti=c(opti$par,logLik=opti$value)
 	re=switch(type[1],
@@ -756,3 +828,14 @@ PlotMixMat=function(mixmat,cutoff.fraction=0.01,mode=c("fill","stack","dodge"))
     ylab("% of reads with k T>C")+xlab("Number of genomic T covered by a read")
 }
 
+
+
+
+# llr: log-likelihood ratio (difference in log-likelihoods between models)
+# Returns: p-value for likelihood ratio test with 1 degree of freedom
+llr.chisq <- function(llr) stats::pchisq(2 * llr, df = 1, lower.tail = FALSE)
+
+# ll0: log-likelihood of the null (simpler/nested) model
+# ll: log-likelihood of the alternative (more complex) model
+# Returns: p-value for likelihood ratio test with 1 degree of freedom
+ll.chisq <- function(ll0, ll) stats::pchisq(-2 * (ll0 - ll), df = 1, lower.tail = FALSE)
